/*
 *     ParallelJ, framework for parallel computing
 *
 *     Copyright (C) 2010 Atos Worldline or third-party contributors as
 *     indicated by the @author tags or express copyright attribution
 *     statements applied by the authors.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package org.parallelj.internal.kernel.split;

import org.parallelj.internal.kernel.KCall;
import org.parallelj.internal.kernel.KProcedure;
import org.parallelj.internal.kernel.KOutputLink;

/**
 * An KXorSplit activates the first {@link KOutputLink} with
 * {@link KOutputLink#verify()} evaluated to <code>true</code>.
 * 
 * If none has been activated, the last one will be activated.
 * 
 * @author Atos Worldline
 */
public class KXorSplit extends KAbstractSplit {

	public KXorSplit(KProcedure procedure) {
		super(procedure);
	}

	@Override
	public void split(KCall call) {
		for (KOutputLink link : this.links) {
			if (link.verify(call.getProcess())) {
				link.activate(call.getProcess());
				return;
			}
		}
		this.forceActivate(call.getProcess());
	}
}
