/*
 *     ParallelJ, framework for parallel computing
 *
 *     Copyright (C) 2010 Atos Worldline or third-party contributors as
 *     indicated by the @author tags or express copyright attribution
 *     statements applied by the authors.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; either
 *     version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package org.parallelj.internal.util;

import org.parallelj.internal.kernel.KProcess;
import org.parallelj.internal.kernel.KProcess.State;

/**
 * Classes that provides helper methods for {@link KProcess}.
 * 
 * @author Atos Worldline
 * 
 */
public privileged aspect KProcesses {

	/**
	 * Property that will store the listeners.
	 * 
	 * @return
	 */
	StateChangeSupport<KProcess, State> KProcess.support;

	after(KProcess process) : execution(KProcess.new(..)) && this(process)  {
		process.support = new StateChangeSupport<KProcess, State>(process);
	}

	Object around(KProcess self) : 
		(call (* KProcess.abort()) ||
		 call (* KProcess.complete()) ||
		 call (* KProcess.fire()) ||
		 call (* KProcess.start()) ||
		 call (* KProcess.terminate())
		) && target(self) {
		KProcess.State oldState = self.state;
		Object o = proceed(self);
		KProcess.State newState = self.state;
		
		if (oldState != newState) {
			this.stateChanged(self, oldState, newState);
		}
		return o;
	}

	protected void stateChanged(KProcess process, State oldState, State newState) {
		process.support.fireStateChanded(process.state);
	}

	public static void addStateListener(KProcess process,
			StateListener<KProcess, State> listener) {
		process.support.addStateListener(listener);
	}

	public void removeStateListener(KProcess process,
			StateListener<KProcess, State> listener) {
		process.support.removeStateListener(listener);
	}

}
